# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/01_transform.ipynb.

# %% auto 0
__all__ = ['AudioLoader', 'Resampling', 'AudioEmbedding', 'Pooling', 'AudioPipeline']

# %% ../nbs/01_transform.ipynb 3
import torch
import numpy as np
import soundfile as sf
from fastcore.all import *
from fasttransform import DisplayedTransform, Pipeline
from transformers import AutoFeatureExtractor

from .core import AudioArray

# %% ../nbs/01_transform.ipynb 7
class AudioLoader(DisplayedTransform):
    """Load audio files into an `AudioArray` object."""
    def __init__(self, sr: int = None): store_attr()
    def encodes(self, x:str) -> AudioArray: return self.load_audio(x, self.sr)

    @staticmethod
    def load_audio(path, sr=None): 
        with sf.SoundFile(path) as f: return AudioArray(f.read(), sr if sr else f.samplerate)

# %% ../nbs/01_transform.ipynb 18
class Resampling(DisplayedTransform):
    """Resample audio to a given sampling rate."""
    def __init__(self, target_sr: int):
        store_attr()
    def encodes(self, audio: AudioArray) -> AudioArray: return self.process_audio_array(audio)
    
    def process_audio_array(self, audio: AudioArray) -> AudioArray:
        if audio.sr == self.target_sr: return audio
        indices = np.linspace(0, len(audio.a) - 1, self._new_length(audio, self.target_sr))
        resampled = np.interp(indices, np.arange(len(audio.a)), audio.a)
        return AudioArray(resampled, self.target_sr)

    def _new_length(self, audio: AudioArray, target_sr: int) -> int:
        return int(len(audio.a) * (target_sr / audio.sr))

# %% ../nbs/01_transform.ipynb 26
class AudioEmbedding(DisplayedTransform):
    """Embed audio using a HuggingFace Audio model."""
    def __init__(self, model_name: str = "MIT/ast-finetuned-audioset-10-10-0.4593", return_tensors: str = "np", **kwargs): 
        store_attr()
        self.model = AutoFeatureExtractor.from_pretrained(model_name, **kwargs)

    def encodes(self, x:AudioArray): return self.call_model(x.a, x.sr)
    
    def call_model(self, x, sr: int):
        return self.model(x, sampling_rate=sr, return_tensors=self.return_tensors)['input_values']

# %% ../nbs/01_transform.ipynb 41
class Pooling(DisplayedTransform):
    """Pool embeddings"""
    def __init__(self, pooling: str):
        assert pooling in ["mean", "max"], "Pooling must be either None (no pooling), 'mean' or 'max'."
        store_attr()

    def encodes(self, x:np.ndarray) -> np.ndarray: 
        if self.pooling == "mean": return x.mean(axis=1)
        elif self.pooling == "max": return x.max(axis=1)

    def encodes(self, x:torch.Tensor) -> torch.Tensor: 
        if self.pooling == "mean": return x.mean(dim=1)[0]
        elif self.pooling == "max": return x.max(dim=1)[0]

# %% ../nbs/01_transform.ipynb 58
class AudioPipeline(Pipeline):
    def __init__(self, 
                 model_name: str = "MIT/ast-finetuned-audioset-10-10-0.4593", 
                 return_tensors: str = "np",
                 target_sr: int = 16_000, 
                 pooling: str = "max", 
                 **kwargs):
        super().__init__([
            AudioLoader(),
            Resampling(target_sr),
            AudioEmbedding(model_name, return_tensors, **kwargs),
            Pooling(pooling)
        ])



# Autogenerated by nbdev

d = { 'settings': { 'branch': 'main',
                'doc_baseurl': '/auditus',
                'doc_host': 'https://CarloLepelaars.github.io',
                'git_url': 'https://github.com/CarloLepelaars/auditus',
                'lib_path': 'auditus'},
  'syms': { 'auditus.core': { 'auditus.core.AudioArray': ('core.html#audioarray', 'auditus/core.py'),
                              'auditus.core.AudioArray.__getitem__': ('core.html#audioarray.__getitem__', 'auditus/core.py'),
                              'auditus.core.AudioArray.__init__': ('core.html#audioarray.__init__', 'auditus/core.py'),
                              'auditus.core.AudioArray.__len__': ('core.html#audioarray.__len__', 'auditus/core.py'),
                              'auditus.core.AudioArray.audio': ('core.html#audioarray.audio', 'auditus/core.py'),
                              'auditus.core.AudioArray.shape': ('core.html#audioarray.shape', 'auditus/core.py')},
            'auditus.transform': { 'auditus.transform.AudioEmbedding': ('transform.html#audioembedding', 'auditus/transform.py'),
                                   'auditus.transform.AudioEmbedding.__init__': ( 'transform.html#audioembedding.__init__',
                                                                                  'auditus/transform.py'),
                                   'auditus.transform.AudioEmbedding.call_model': ( 'transform.html#audioembedding.call_model',
                                                                                    'auditus/transform.py'),
                                   'auditus.transform.AudioEmbedding.encodes': ( 'transform.html#audioembedding.encodes',
                                                                                 'auditus/transform.py'),
                                   'auditus.transform.AudioLoader': ('transform.html#audioloader', 'auditus/transform.py'),
                                   'auditus.transform.AudioLoader.__init__': ( 'transform.html#audioloader.__init__',
                                                                               'auditus/transform.py'),
                                   'auditus.transform.AudioLoader.encodes': ('transform.html#audioloader.encodes', 'auditus/transform.py'),
                                   'auditus.transform.AudioLoader.load_audio': ( 'transform.html#audioloader.load_audio',
                                                                                 'auditus/transform.py'),
                                   'auditus.transform.AudioPipeline': ('transform.html#audiopipeline', 'auditus/transform.py'),
                                   'auditus.transform.AudioPipeline.__init__': ( 'transform.html#audiopipeline.__init__',
                                                                                 'auditus/transform.py'),
                                   'auditus.transform.Pooling': ('transform.html#pooling', 'auditus/transform.py'),
                                   'auditus.transform.Pooling.__init__': ('transform.html#pooling.__init__', 'auditus/transform.py'),
                                   'auditus.transform.Pooling.encodes': ('transform.html#pooling.encodes', 'auditus/transform.py'),
                                   'auditus.transform.Resampling': ('transform.html#resampling', 'auditus/transform.py'),
                                   'auditus.transform.Resampling.__init__': ('transform.html#resampling.__init__', 'auditus/transform.py'),
                                   'auditus.transform.Resampling._new_length': ( 'transform.html#resampling._new_length',
                                                                                 'auditus/transform.py'),
                                   'auditus.transform.Resampling.encodes': ('transform.html#resampling.encodes', 'auditus/transform.py'),
                                   'auditus.transform.Resampling.process_audio_array': ( 'transform.html#resampling.process_audio_array',
                                                                                         'auditus/transform.py')}}}



"""Audio Embeddings"""

# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/00_core.ipynb.

# %% auto 0
__all__ = ['AudioArray']

# %% ../nbs/00_core.ipynb 3
import numpy as np
from fastcore.all import *

# %% ../nbs/00_core.ipynb 5
class AudioArray(BasicRepr): 
    """Representation for audio data."""
    def __init__(self, a: np.ndarray, sr: int): store_attr()
    @property
    def shape(self): return self.a.shape
    def audio(self): 
        from IPython.display import Audio
        return Audio(self.a, rate=self.sr)
    def __len__(self): return len(self.a)
    def __getitem__(self, idx): return self.a[idx]
